# This file is part of SSHare.
#
# SSHare is free software: you can redistribute it and/or modify it under the terms of
# the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# SSHare. If not, see <https://www.gnu.org/licenses/>.

import argparse
import getpass
import os
import time
import tomllib
import subprocess
import sys

from sshare                  import config_directory
from sshare.logger           import Logger
from sshare.plugin           import Plugin
from sshare.plugin           import PluginManager
from sshare.version          import version

def main():
    arg_parser = argparse.ArgumentParser(
        prog="sshare",
        description="Upload files to a server via ssh",
        add_help=False,
    )
    arg_parser.add_argument(
        "--config",
        metavar="config",
        help="Specify location of config file to use"
    )
    arguments, _ = arg_parser.parse_known_args()
    with open(arguments.config or config_directory.default_config(), mode="rb") as file:
        config = tomllib.load(file)
    config["config"] = config.get("config", {})
    config["flags" ] = config.get("flags",  {})

    arg_parser.add_argument(
        "-h",
        "--help",
        action="help",
        help="show this help message and exit"
    )
    arg_parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"%(prog)s version {version}",
    )

    # Load command line early and set it as the active logger
    # so that it can be used to report errors while loading and
    # configuring other loggers
    logger = Logger()
    command_line = Plugin.internal(
        "command_line",
        logger,
        config["config"],
        config["flags"],
    )
    logger.add(command_line)
    plugins = PluginManager(
        logger,
        config["config"],
        config["flags"],
        arg_parser,
    )
    plugins.activate("logger")
    logger.add(*plugins.logger.active)
    plugins.activate()

    for plugin_type in [ "source", "name", "upload" ]:
        plugins_of_type = getattr(plugins, plugin_type)
        if len(plugins_of_type.active) == 0:
            logger.error(f"Error: No '{plugin_type}' plugins activated. Available plugins:")
            for plugin in plugins_of_type.inactive:
                logger.error(f" => {plugin.name}")
            sys.exit(1)
    if len(plugins.location.active) == 0 and len(plugins.feedback.active) > 0:
        logger.warn("Warning: 'feedback' plugins activated with no active 'location' plugins")

    sources = []
    for plugin in plugins.source.active:
        sources.append(plugin.get_source())

    for index, source in enumerate(sources):
        name = ""
        for plugin in plugins.name.active:
            name = plugin.get_name(name, source)
        sources[index] = name, source

    for name, source in sources:
        for plugin in plugins.upload.active:
            plugin.upload(name, source)

    for index, (name, _) in enumerate(sources):
        for plugin in plugins.location.active:
            sources[index] = plugin.get_location(name)

    for location in sources:
        for plugin in plugins.feedback.active:
            sources[index] = plugin.give_feedback(location)

    sys.exit(0)
