# This file is part of SSHare.
#
# SSHare is free software: you can redistribute it and/or modify it under the terms of
# the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# SSHare. If not, see <https://www.gnu.org/licenses/>.

import importlib
import importlib.util
import sys

from pathlib import Path

from sshare.plugin.config import Flag
from sshare.plugin.config import NoDefault
from sshare               import config_directory

class PluginLoader:
    @staticmethod
    def all(command_line=False, logger=None, config=dict(), flags=dict()):
        return PluginLoader.internal(command_line, logger, config, flags) + PluginLoader.external(logger, config, flags)
    
    @staticmethod
    def internal(command_line=False, logger=None, config=dict(), flags=dict()):
        return [
            Plugin.internal(plugin, logger, config, flags)
            for plugin
            in ([ "command_line" ] if command_line else []) + [
                "file",
                "stdin",
                "current_time",
                "append_type",
                "ssh",
                "uri",
                "print_location",
            ]
        ]

    @staticmethod
    def external(logger=None, config=dict(), flags=dict()):
        return [
            Plugin.external(plugin, logger, config, flags)
            for plugin
            in config_directory.plugins()
        ]

    @staticmethod
    def at(*args, logger=None, config=dict(), flags=dict()):
        return [
            Plugin.external(Path(plugin), logger, config, flags)
            for plugin
            in args
        ]

class PluginManager:
    def __init__(self, logger, config, flags, arg_parser):
        self._logger        = logger
        self._arg_parser    = arg_parser

        class PluginState:
            def __init__(self):
                self.active = []
                self.inactive = []
        for type in Plugin.types():
            setattr(self, type, PluginState())

        self._uninitialized = PluginLoader.all(
            command_line=False,
            logger=logger,
            config=config,
            flags=flags,
        )
        for plugin in self._uninitialized:
            plugin.add_args(arg_parser)

    def activate(self, activate_type=None):
        args = self._arg_parser.parse_args()
        for plugin in self._uninitialized.copy():
            if activate_type == None or activate_type in plugin.plugin_type:
                self._uninitialized.remove(plugin)
                active = plugin.load_args_and_activate(args)
                for type in plugin.plugin_type:
                    getattr(
                        getattr(self, type),
                        active[type],
                    ).append(plugin)

class Plugin:
    @staticmethod
    def types(methods=False):
        types = {
            "logger":   { "info", "warn", "error" },
            "source":   { "get_source" },
            "name":     { "get_name" },
            "upload":   { "upload" },
            "location": { "get_location" },
            "feedback": { "give_feedback"},
        }
        if methods:
            return types
        else:
            return types.keys()

    def __init__(self, name, module, logger, external_config, external_flags):
        self.__dict__ = module.__dict__
        self.name     = name

        if logger == None:
            return

        self.logger = logger

        if not isinstance(self.plugin_type, set):
            self.plugin_type = { self.plugin_type }

        if hasattr(self, "activate"):
            if not isinstance(self.activate, dict):
                activate = self.activate
                self.activate = dict()
                for plugin_type in self.plugin_type:
                    self.activate[plugin_type] = activate
        else:
            self.activate = dict()
            for plugin_type in self.plugin_type:
                self.activate[plugin_type] = set()

        if hasattr(self, "config"):
            config = self.config
        else:
            config = dict()
        if external_config == None:
            external_config = dict()
        for key in config.keys():
            if key in external_config:
                config[key] = external_config[key]

        if hasattr(self, "args"):
            for arg in self.args.items():
                arg[1].bind(self, arg[0])
                flags = external_flags.get(arg[0], dict())
                arg[1].set_flags(flags.get("short"), flags.get("long"))
                value = arg[1].default()
                if value != NoDefault:
                    config[arg[0]] = value

        else:
            self.args = dict()

        class Config: pass
        flat_config = Config()
        flat_config.__dict__ = config
        self.config = flat_config

    def add_args(self, arg_parser):
        for arg in self.args.values():
            arg.add(arg_parser)

    def load_args_and_activate(self, args):
        passed_args = set()
        for arg_name, arg in self.args.items():
            was_set, value = arg.extract(args)
            if was_set:
                if value != Flag:
                    setattr(self.config, arg_name, value)
                passed_args.add(arg_name)
        activate = dict()
        run_init = False
        for type in self.plugin_type:
            if self.activate[type] <= passed_args:
                activate[type] = "active"
                run_init = True
            else:
                activate[type] = "inactive"
        if run_init and hasattr(self, "init"):
            self.init()
        return activate
    
    @staticmethod
    def internal(name=None, logger=None, config=dict(), flags=dict()):
        return Plugin(name, importlib.import_module(f"sshare.plugins.{name}"), logger, config.get(name, dict()), flags.get(name, dict()))

    @staticmethod
    def external(path, logger=None, config=dict(), flags=dict()):
        sys.dont_write_bytecode = True
        module_spec = importlib.util.spec_from_file_location(
            path.stem,
            path.as_posix(),
        )
        module = importlib.util.module_from_spec(module_spec)
        module_spec.loader.exec_module(module)
        sys.dont_write_bytecode = False
        return Plugin(path.stem, module, logger, config.get(path.stem, dict()), flags.get(path.stem, dict()))
